/*****************************************************************
 *
 *  Copyright (c) NEC America, Inc. 2000-2005
 *  All Rights Reserved
 *
 *  No part of this program may be photocopied, reproduced, or
 *  translated to another programming language without the prior
 *  written consent of NEC America, Inc.
 *
 *****************************************************************/

import com.nec.tdd.tools.dbMapper.DataSource;
import com.nec.tdd.tools.dbMapper.DBConnection;
import com.nec.tdd.tools.dbMapper.DBConnectionImpl;
import com.nec.tdd.tools.logger.Logger;
import java.sql.*;


public class MyDataSource implements DataSource {

  /**
   * Please refer to JDBC API for DriverManager.getConnection(String,Properties)
   * method.
   * A list of arbitrary string tag/value pairs as connection arguments;
   * normally at least a "user" and "password" property should be included.
   */
   java.util.Properties connectionArgs = null;

  /**
   * Following constructor signature is required by dbMapper API for custom
   * data-source objects.
   * @param    props A list of string/value pairs as specified in dbmapper
   *                 config XML file (<property> tags) for this data-source.
   */
   public MyDataSource (java.util.Properties props) throws Exception {
      String jdbcDriverClassName = props.getProperty ("driver");
      connectionArgs = props;

      // Load the JDBC driver
      Class.forName (jdbcDriverClassName);
   }

  /*********** Implementation of DataSource methods ************/

  /**
   * Get a DBConnection.
   *
   * @return a new JDBC connection wrapped inside DBConnection.
   */
   public DBConnection getConnection() {
      DBConnection dc = null;

      String jdbcURL = connectionArgs.getProperty ("url");
      try {
         java.sql.Connection conn = DriverManager.getConnection (jdbcURL,
                                                                connectionArgs);
         dc = new DBConnectionImpl (conn);
      }
      catch (Exception e) {
         Logger.error ("Error creating connection. Details: ", e);
      }
      return dc;
   }


  /**
   * Method to release an previously opened DBConnection.
   *
   * @param conn DBConnection to be released.
   */
   public void releaseConnection (DBConnection conn) {
      ((DBConnectionImpl) conn).destroy();
   }

  /**
   * Returns string representation of MyDataSource.
   */
   public String toString () {
      return "MyDataSource: connectionArgs=" + connectionArgs;
   }

}
