import java.util.*;
import com.nec.tdd.tools.dbMapper.*;
import com.nec.tdd.tools.logger.Logger;


public class Test {

   DBInterface mapper; // mapper used for this test

   Test (String mapperFile) {

      // create the mapper to be used for this test
      try {
         DBModule dbm = DBModule.init (mapperFile);
         mapper = dbm.createDefaultMapper ("default");
         Logger.debug (mapper.toString());
      }
      catch (Exception e) {
         Logger.error ("Error details: ", e);
         System.exit(0);
      }
      this.mapper = mapper;
   }

   public static void main (String[] args) {
      try {
         String mapperFile = args[0];
         Test testSuite = new Test(mapperFile);
         testSuite.test();
      }
      catch (Exception e) {
         Logger.error ("Error details: ", e);
      }
   }

   void test() throws Exception {
      AttrValMap aValMap = null;
      HashMap    hValMap = null;

      // Delete older entries
      mapper.deleteAll (Primitive.class);

      // Create a Primitive object and persist it to database
      Primitive o1 = new Primitive(3, "test");
      mapper.create (o1);
      Logger.debug ("create(Object) :: Object " + o1
                   + " is successfully created.");

      // Delete the o1 Primitive object created above
      mapper.delete (o1);
      Logger.debug ("delete(Object) :: Object " + o1
                    + " is successfully deleted.");

      // Create o1 again and then delete it using primay key
      mapper.create (o1);
      mapper.deleteByPrimaryKey (new Integer(3), Primitive.class);
      Logger.debug ("delete(pk,cls) :: Object " + o1
                  + " is successfully deleted.");

      // Create o1 again. Delete all the Primitive objects with x equal
      // to 3 (deletes o1 from database)
      mapper.create (o1);
      aValMap = new AttrValMap();
      aValMap.put ("x", 3);
      mapper.deleteByAttributes (aValMap, Primitive.class);
      Logger.debug ("deleteByAttributes(aval,cls) :: Object " + o1
                   + " is successfully deleted.");

      // Create o1 again. Update o1 attributes and update new changes
      // to database
      mapper.create (o1);
      Logger.debug ("create(Object) :: Object " + o1
                   + " is successfully created.");
      o1.setY ("New Value");
      mapper.update(o1);
      Logger.debug ("update(Object) :: Object " + o1
                   + " is successfully updated.");


      // Update selected o1 attributes to database (using AttrValMap)
      aValMap = new AttrValMap();
      aValMap.put ("y", "AttrVal update");
      mapper.update(o1, aValMap, true);
      Logger.debug ("update(Object,aval,bool) :: Object " + o1
                   + " is successfully updated.");

      // Update selected o1 attributes to database (using HashMap)
      hValMap = new HashMap(1);
      hValMap.put ("y", "HashMap update");
      mapper.update(o1, hValMap, true);
      Logger.debug ("update(Object,hval,bool) :: Object " + o1
                   + " is successfully updated.");


      // Create few more Primitive objects and persist them to database
      Primitive o2 = new Primitive(6, null);
      mapper.create (o2);
      Primitive o3 = new Primitive(36, "Welcome !");
      mapper.create (o3);
      Primitive o4 = new Primitive(36*6, "Hi");
      mapper.create (o4);


      // Locate an previously created Primitive object in database using
      // primary key
      Primitive retVal = (Primitive) mapper.findByPrimaryKey (
                  new Integer(36), Primitive.class); // should return o3
      Logger.debug ("findByPrimaryKey(pk,class) :: Object " + retVal
                   + " is successfully found.");


      // Locate an previously created Primitive object in database by
      // specifying key attribute x
      aValMap = new AttrValMap();
      aValMap.put ("x", 36*6);
      Collection c1 = mapper.findByAttributes (aValMap, Primitive.class);
      // c1 should contain only o4
      Logger.debug ("findByAttributes(aval,class) :: Collection "
                     + c1.toString() + " is successfully found.");


      // Find all the Primitive object in database with null y (non-key
      // attribute)
      aValMap = new AttrValMap();
      aValMap.put ("y", null);
      Collection c2 = mapper.findByAttributes (aValMap, Primitive.class);
      // c2 hould contain only o2
      Logger.debug ("findByAttributes(aval,class) :: Collection "
                     + c2.toString() + " is successfully found.");

      // Find Primitive objects based on a custom SQL query
      Collection c3 = mapper.findByQuery (
               "SELECT * from demo_primitive where col_x >= 36 order by col_x",
               Primitive.class);
      Logger.debug ("findByQuery(query,class) :: Collection "
                     + c3.toString() + " is successfully found.");

      // Find all the Primitive objects stored in demo_primitive table
      Collection c4 = mapper.findAll (Primitive.class);
      Logger.debug ("findAll(class) :: Collection " + c4.toString()
                    + " is successfully found.");

      // Find a primary key object based on non-key attribute y.
      // And then delete the corresponding object from database.
      aValMap = new AttrValMap();
      aValMap.put ("y", "Hi");
      Collection c5 = mapper.findPrimaryKeysByAttributes (
                                                aValMap, Primitive.class);
      // Above method should return o4. Remove it
      Logger.debug ("findPrimaryKeysByAttributes(aval,class) :: Collection "
                    + c5.toString() + " is successfully found.");
      Object akey = c5.iterator().next();
      mapper.deleteByPrimaryKey (akey, Primitive.class);

      // Find primary key objects based on a custom SQL query
      Collection c6 = mapper.findPrimaryKeysByQuery (
                  "SELECT col_x from demo_primitive where col_y IS NOT NULL",
                        Primitive.class);
      Logger.debug ("findPrimaryKeysByQuery(query,class) :: Collection "
                   + c6.toString() + " is successfully found.");

      // Find primary key of all the Primitive objects stored in
      // demo_primitive table
      Collection c7 = mapper.findAllPrimaryKeys (Primitive.class);
      Logger.debug ("findAllPrimaryKeys(class) :: Collection "
                   + c7.toString() + " is successfully found.");

      // Add "36*6" back.
      mapper.create (o4);
      Collection c8 = mapper.findAll (Primitive.class);
      Logger.debug ("findAll(class) :: Collection " + c8.toString()
                    + " is successfully found.");

   }

}

