import java.io.Serializable;
import com.nec.tdd.tools.dbMapper.*;
import java.util.*;
import javax.ejb.*;

public class CounterEJB implements EntityBean {
   private EntityContext ctx;
   private CounterBean counter = new CounterBean();
   private DBInterface mapper = null;

   public void setEntityContext(EntityContext ctx) {
      this.ctx = ctx;
   }

   public void unsetEntityContext() {
      this.ctx = null;
   }

   public void ejbActivate() {
      System.out.println ("ejbActivate (" + ctx.getPrimaryKey() + ")");
   }

   public void ejbPassivate() {
   }

   /**
    * Loads the EJBean from the persistent storage.
    */
   public void ejbLoad() {
      String id = (String) ctx.getPrimaryKey();
      System.out.println ("ejbLoad (" + id + ")");
      try {
         initDBIf();
         CounterBean newCounter =
                  (CounterBean) mapper.findByPrimaryKey (id, CounterBean.class);
         if (null == counter) {
            throw new NoSuchEntityException ("ejbLoad: No counter with id="
                                             + id);
         }
         counter = newCounter;
      } catch (Exception ex) {
         throw new EJBException(ex);
      }
   }

   /**
    * Stores the EJBean in the persistent storage.
    */
   public void ejbStore() {
      String id = (String) ctx.getPrimaryKey();
      System.out.println ("ejbStore (" + id + ")");
      try {
         initDBIf();
         mapper.update(counter);
      } catch (Exception ex) {
         throw new EJBException(ex);
      }
   }

   public String ejbCreate (String counterId) throws CreateException {
      System.out.println ("ejbCreate (" + counterId + ")");
      counter.setCounterId(counterId);
      counter.setValue(0);

      try {
         initDBIf();
         mapper.create(counter);
      } catch (Exception ex) {
         throw new CreateException (ex.getMessage());
      }

      return counterId;
   }

   public void ejbPostCreate(String counterId) {
   }

   /**
    * Deletes the EJBean from the persistent storage.
    */
   public void ejbRemove() {
      String id = (String) ctx.getPrimaryKey();
      System.out.println ("ejbRemove (" + id + ")");
      boolean isDeleted = true;
      try {
         initDBIf();
         isDeleted = mapper.deleteByPrimaryKey (id, CounterBean.class);
      } catch (Exception ex) {
         throw new EJBException(ex);
      }
      if (! isDeleted) {
         throw new NoSuchEntityException ("No counter with id " + id);
      }
   }

   /**
    * Attempts to find the EJBean with a given Primary Key from
    * the persistent storage.
    */
   public String ejbFindByPrimaryKey (String pk)
                                  throws ObjectNotFoundException {
      System.out.println ("ejbFindByPrimaryKey (" + pk + ")");
      CounterBean bean = null;
      try {
         initDBIf();
         bean = (CounterBean) mapper.findByPrimaryKey (pk, CounterBean.class);
      } catch (Exception ex) {
         throw new EJBException (ex);
      }
      if (bean != null) {
         System.out.println ("ejbFindByPrimaryKey found counter[" + pk + "]");
         counter = bean;
      } else {
         throw new NoSuchEntityException ("No counter with id " + pk);
      }
      return pk;
   }

   public Enumeration ejbFindAll() {
      System.out.println ("ejbFindAll");
      try {
         initDBIf();
         Collection c = mapper.findAllPrimaryKeys (CounterBean.class);
         System.out.println ("The Collection is " + c);
         return Collections.enumeration (c);
      } catch (Exception ex) {
         throw new EJBException (ex);
      }
   }

   public Enumeration ejbFindNonZeroCounters() {
      System.out.println ("ejbFindNonZeroCounters");
      try {
         initDBIf();
         Collection c = mapper.findPrimaryKeysByQuery (
               "select id from demo_counter where value<>0", CounterBean.class);
         return Collections.enumeration (c);
      } catch (Exception ex) {
         throw new EJBException (ex);
      }
   }

   public int increment () {
      System.out.println("Incrementing counter[" + counter.getCounterId() +"]");
      return counter.increment();
   }

   public int decrement () {
      System.out.println("Decrementing counter[" + counter.getCounterId() +"]");
      return counter.decrement();
   }

   public int value() {
      return counter.getValue();
   }

   private void initDBIf () throws Exception {
      if (mapper != null) {
         return;
      }

      // Get the shared singelton DBModule instance
      System.out.println ("Loading DB Mapper file and creating the mapping "
                         + "context");
      DBModule dbm = DBModule.init ("/dbmapper.xml");
      mapper = dbm.createDefaultMapper ("default");

/*
      // Uncomment following if dont want to use XML files
      // Following code creates mapper using dbMapper classes directy (refer to
      // javadoc API for details)
      DBModule dbm = DBModule.init();
      ORMapEntry mapEntry = new ORMapEntry();
      mapEntry.setClassName (CounterBean.class.getName());
      mapEntry.setTableName ("demo_counter");
      ORFieldInfo[] fields = new ORFieldInfo [2];
      fields[0] = new ORFieldInfo (mapEntry, "counterId", null, null, true,
                     new ORFieldInfo.BasicTypeInfo("String", "id"));
      fields[1] = new ORFieldInfo (mapEntry, "value", null, null, false,
                     new ORFieldInfo.BasicTypeInfo("int", "value"));
      mapEntry.setFields (fields);
      ORMappingInfo mappingInfo = new ORMappingInfo();
      mappingInfo.add (mapEntry);

      com.nec.tdd.tools.dbMapper.JNDIDataSource ds =
            new com.nec.tdd.tools.dbMapper.JNDIDataSource ("jdbc/OracleCoreDS");

      mapper = new DefaultMapper (ds, mappingInfo);
*/
   }

}

