import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import com.nec.tdd.tools.dbMapper.DBInterface;
import com.nec.tdd.tools.logger.Logger;

public class PointPanel extends JPanel
      implements MouseListener, MouseMotionListener, ComponentListener {
   static Color defaultBkColor = new Color(205, 223, 255);
   static Color dragBkColor    = new Color(144, 188, 255);
   static Color createBkColor  = new Color(223, 133, 133);
   static Color findBkColor    = new Color(200, 238, 195);

   Point p;
   DBInterface dbIf;
   int lastClickX, lastClickY;

   public PointPanel (DBInterface dbIf, Point p) {
      super ();
      this.p = p;
      this.dbIf = dbIf;
      setBackground (defaultBkColor);
      Border innerBorder = BorderFactory.createCompoundBorder (
                                 BorderFactory.createLoweredBevelBorder(),
                                 BorderFactory.createRaisedBevelBorder());
      Border b = BorderFactory.createTitledBorder (innerBorder, ""+p.getId(),
                                 TitledBorder.CENTER, TitledBorder.TOP,
                                 new Font("Dialog", Font.BOLD, 12));
      this.setBorder (b);
      this.setToolTipText (p.toString());
      movePoint();

      addMouseListener (this);
      addMouseMotionListener (this);
   }

   public Point getPoint() {
      return p;
   }

   public void setCreateColor () {
      this.setBackground (createBkColor);
   }

   public void setFindColor () {
      this.setBackground (findBkColor);
   }

   public void resetColor () {
      this.setBackground (defaultBkColor);
   }

   public void movePoint () {
      Dimension dim = this.getPreferredSize();
      this.setBounds (p.getX(), p.getY(), dim.width, dim.height);
   }

   // MouseListener
   public void  mouseClicked(MouseEvent e) {
   /*
      if ((e.getModifiers() & InputEvent.BUTTON1_MASK)
                == InputEvent.BUTTON1_MASK ) {
         System.out.println ("Selected point = " + p.getId());
         popupMenu.show (this, e.getX(), e.getY());
      }
    */
   }

   public void  mouseEntered(MouseEvent e){}
   public void  mouseExited(MouseEvent e){}

   public void  mousePressed(MouseEvent e) {
      this.setBackground (dragBkColor);
      lastClickX = e.getX();
      lastClickY = e.getY();
   }

   public void  mouseReleased(MouseEvent e) {
      this.setBackground (defaultBkColor);
      try {
         dbIf.update (p);
      } catch (Exception ex) {
         Logger.error (ex.toString());
      }
      this.setToolTipText (p.toString());
   }





   // MouseMotionListener
   public void  mouseDragged(MouseEvent e) {
      p.setX (p.getX() + e.getX() - lastClickX);
      p.setY (p.getY() + e.getY() - lastClickY);
      // System.out.println ("Finally currX=" + p.getX() + " currY=" + p.getY());
      movePoint ();
   }

   public void  mouseMoved(MouseEvent e) {
   }


   // ComponentListener
   public void componentResized(ComponentEvent e) {
   }
   public void componentMoved(ComponentEvent e) {}
   public void componentShown(ComponentEvent e) {}
   public void componentHidden(ComponentEvent e) {}
}
